-- added new fields to support both types of items in the catalog
-- by default, existing catalog items are of type service (itemtype=0) and billed in units (quantitytype=0)
ALTER TABLE service ADD itemtype INTEGER;
ALTER TABLE service ADD quantitytype INTEGER;

UPDATE service SET itemtype=0,quantitytype=0;

-- 1. items represent both services and expenses
-- added new fields to support both types of items
--
-- itemtype = {0 = services, 1 = expenses}
-- quantitytype = {0 = unit-based, 1 = hours}
-- billing = {0 = none, 1 = flat, 2 = hourly}

ALTER TABLE expense ADD taxexempt BOOLEAN;

ALTER TABLE item ADD notes VARCHAR(250);
ALTER TABLE item ADD taxexempt BOOLEAN;
ALTER TABLE item ADD itemtype INTEGER;
ALTER TABLE item ADD billable BOOLEAN;
ALTER TABLE item ADD quantitytype INTEGER;

-- 1.1. update new fields
UPDATE item SET notes='',taxexempt=FALSE;
UPDATE item SET itemtype=0;
UPDATE item SET billable=FALSE,quantitytype=1 WHERE billing=0;
UPDATE item SET billable=TRUE,quantitytype=0 WHERE billing=1;
UPDATE item SET billable=TRUE,quantitytype=1 WHERE billing=2;

-- 1.2. reset old field
UPDATE item SET billing=null;

-- 2. move expenses to the item table
-- 2.1. move invoiced expenses
INSERT INTO item (projectid,name,description,notes,date,itemtype,billable,taxexempt,units,invoiceprojectid,quantitytype,cost,costcurrency) 
SELECT projectid,name,description,attachment,date,1,TRUE,TRUE,1.0,invoiceprojectid,0,amount,currency
FROM expense
WHERE expense.invoiceprojectid != 0;

-- 2.2. move uninvoiced expenses
INSERT INTO item (projectid,name,description,notes,date,itemtype,billable,taxexempt,units,invoiceprojectid,quantitytype,cost,costcurrency) 
SELECT projectid,name,description,attachment,date,1,TRUE,TRUE,1.0,null,0,amount,currency
FROM expense
WHERE expense.invoiceprojectid = 0;

-- 2.3. make sure notes are set to an empty string; some expenses may have a null attachment and that means a null notes field when copied to the item table
UPDATE item SET notes='' WHERE notes=null;

-- 2.4. delete expenses once they are moved to the item table
DELETE FROM expense;