-- create new product table
CREATE TABLE product (
	productid BIGINT GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	uuid VARCHAR(36) NOT NULL,
	created TIMESTAMP NOT NULL,
	updated TIMESTAMP,
	projectid BIGINT,
	itemid BIGINT,
	name VARCHAR(255) NOT NULL,
	catalogitemid BIGINT,
	description VARCHAR(2000),
	taxexempt BOOLEAN,
	regularquantity NUMERIC,
	quantitydiscountenabled BOOLEAN,
	quantitydiscounttype INTEGER,
	quantitydiscountpercentagevalue NUMERIC,
	quantitydiscountfixedvalue NUMERIC,
	unitofmeasure VARCHAR(50),
	pricediscountenabled BOOLEAN,
	pricediscounttype INTEGER,
	pricediscountpercentagevalue NUMERIC,
	pricediscountamount NUMERIC,
	pricediscountcurrency VARCHAR(255),
	amount NUMERIC,
	currency VARCHAR(255),
	PRIMARY KEY(productid),
	CONSTRAINT fk_product_projectid FOREIGN KEY(projectid) REFERENCES project(projectid),
	CONSTRAINT fk_product_itemid FOREIGN KEY(itemid) REFERENCES item(itemid),
	CONSTRAINT fk_product_catalogitemid FOREIGN KEY(catalogitemid) REFERENCES catalogitem(catalogitemid)
);

-- add a new product column to the item table
ALTER TABLE item ADD productid BIGINT;
ALTER TABLE item ADD CONSTRAINT fk_item_productid FOREIGN KEY (productid) REFERENCES product(productid);

-- copy project items to the new product table
INSERT INTO product (productid,uuid,created,updated,projectid,name,catalogitemid,description,taxexempt,regularquantity,quantitydiscountenabled,quantitydiscounttype,quantitydiscountpercentagevalue,quantitydiscountfixedvalue,unitofmeasure,pricediscountenabled,pricediscounttype,pricediscountpercentagevalue,pricediscountamount,pricediscountcurrency,amount,currency) 
SELECT itemid,uuid,created,updated,projectid,name,catalogitemid,description,taxexempt,units,unitsdiscountenabled,unitsdiscounttype,unitsdiscountpercentagevalue,unitsdiscountfixedvalue,unitofmeasure,ratediscountenabled,ratediscounttype,ratediscountpercentagevalue,ratediscountrate,ratediscountcurrency,rate,currency
FROM item
WHERE item.projectid is not null;

-- link invoiced products to their items
UPDATE product SET itemid=(SELECT item.itemid FROM item WHERE product.productid=item.itemid AND item.invoiceid is not null AND item.projectid is not null);
UPDATE item SET productid=itemid WHERE invoiceid is not null AND projectid is not null;

-- remove all not invoiced product items because they were replaced by products
DELETE FROM item WHERE invoiceid is null AND projectid is not null;

-- drop project column for item because it's not used anymore
ALTER TABLE item DROP CONSTRAINT fk_item_projectid;
ALTER TABLE item DROP COLUMN projectid;